BuffFrame:Hide()

local NUM_DEBUFFS = 20
local BuffModule = Dominos:NewModule('Buffs')
local BuffFrame = Dominos:CreateClass('Frame', Dominos.Frame)
local LBF = LibStub('LibButtonFacade', true)
local UNIT = "player"

--[[ Module Code ]]--

function BuffModule:Load()
	self.frame = BuffFrame:New()
end

function BuffModule:Unload()
	self.frame:Free()

end

--[[ Buff Frame Code ]]--
function BuffFrame:New()

	local f = self.super.New(self, 'buffs')
	local buffs 
	if not self.buffs then
		buffs  = f:CreateBuffs()
		f:SetFrameStrata('LOW')
	end

	f:LoadButtons()
	f:Layout()
	return f

end

function BuffFrame:GetDefaults()
	return {
		scale = 1,
		point = 'CENTER',
		numButtons = NUM_DEBUFFS,
		y = 0,
		x = 0,
	}

end

function BuffFrame:NumButtons()
	return self.sets.numButtons or NUM_DEBUFFS
end

function BuffFrame:AddButton(i)
	local b = self.buffs[i]
	b:SetParent(self.header)
	self.buttons[i] = b
end

function BuffFrame:RemoveButton(i)
	local b = self.buttons[i]
	b:SetParent(nil)
	b:Hide()
	self.buttons[i] = nil
end


function BuffFrame:CreateBuffs()
	if not self.buffs then
		self.buffs = {}
	end
	for i = 1, NUM_DEBUFFS do
		if not self.buffs[i] then
			self.buffs[i] = self:CreateBuff(i)
		end
	end
end
function BuffFrame:CreateBuff(index)
	local frame, frameName;
	local selfName = "DominosBuff"
	local frameName = selfName..index;
	local buff = CreateFrame("Button", frameName, UIParent, "BuffButtonTemplate, TargetBuffFrameTemplate")
	buff:SetSize(34, 34)
	buff.icon = _G[frameName.."Icon"]
	buff.icon:SetTexCoord(.1,.9,.1,.9)
	buff:SetScript("OnClick", function() CancelUnitBuff(UNIT, index, "HELPFUL") end)

	buff.cooldown = _G[frameName.."Cooldown"]
	self:SetScript("OnUpdate", function(self, elapsed)
		for i=1, self:NumButtons() do
			self:UpdateBuff(i)
		end
	end)
	return buff
end

function BuffFrame:UpdateBuff(index, elapsed)
	local buff = self.buffs[index]
	BuffFrame:AuraButtonUpdate(buff:GetName(), index, "HELPFUL")
	buff.id = index
end


--[[ Menu Code ]]--

local function AddSizeSlider(p)
	local L = LibStub('AceLocale-3.0'):GetLocale('Dominos-Config')
	local size = p:NewSlider(L.Size, 1, 1, 1)
	size.OnShow = function(self)
		self:SetMinMaxValues(1, NUM_DEBUFFS)
		self:SetValue(self:GetParent().owner:NumButtons())
	end
	size.UpdateValue = function(self, value)
		self:GetParent().owner:SetNumButtons(value)
		_G[self:GetParent():GetName() .. L.Columns]:OnShow()
	end
end


local function AddLayoutPanel(menu)
	local p = menu:NewPanel(LibStub('AceLocale-3.0'):GetLocale('Dominos-Config').Layout)
	p:NewOpacitySlider()
	p:NewFadeSlider()
	p:NewScaleSlider()
	p:NewPaddingSlider()
	p:NewSpacingSlider()
	p:NewColumnsSlider()
	AddSizeSlider(p)
end

local function AddAdvancedLayout(self)
	self:AddAdvancedPanel()
end


function BuffFrame:CreateMenu()
	local menu = Dominos:NewMenu(self.id)
	AddLayoutPanel(menu)
	AddAdvancedLayout(menu)
	self.menu = menu
end

function BuffFrame:AuraButtonUpdate(buttonName, index, filter)
	local unit = UNIT;
	local name, rank, texture, count, debuffType, duration, expirationTime = UnitAura(unit, index, filter);
	local buffName = buttonName;
	local buff = _G[buffName];

	if ( not name ) then
		-- No buff so hide it if it exists
		if ( buff ) then
			buff:Hide();
			buff.duration:Hide();
		end
		return nil;
	else
		-- Setup Buff
		buff:SetID(index);
		buff.unit = unit;
		buff.filter = filter;
		buff:SetAlpha(1.0);
		buff.exitTime = nil;
		buff:Show();

		if ( duration > 0 and expirationTime ) then
			buff.cooldown:Show()
			CooldownFrame_SetTimer(buff.cooldown, expirationTime - duration, duration, 1)
		else
			buff.cooldown:Hide()
		end
		if ( duration > 0 and expirationTime ) then
			if ( SHOW_BUFF_DURATIONS == "1" ) then
				buff.duration:Show();
			else
				buff.duration:Hide();
			end
			if ( not buff.timeLeft ) then
				buff.timeLeft = expirationTime - GetTime();
				buff:SetScript("OnUpdate", AuraButton_OnUpdate);
			else
				buff.timeLeft = expirationTime - GetTime();
			end		 
			buff.expirationTime = expirationTime; 
		else
			buff.duration:Hide();
			if ( buff.timeLeft ) then
				buff:SetScript("OnUpdate", nil);
			end
			buff.timeLeft = nil;
		end
		local icon = _G[buffName.."Icon"];
		icon:SetTexture(texture);
 
		if ( count > 1 ) then
			buff.count:SetText(count);
		else
			buff.count:SetText("");
		end
 
		if ( GameTooltip:IsOwned(buff) ) then
			GameTooltip_SetDefaultAnchor(GameTooltip, buff)
			GameTooltip:SetUnitAura(UNIT, index, filter);
		end
	end
	return 1;
end