local NUM_DEBUFFS = 20
local DebuffModule = Dominos:NewModule('Debuffs')
local DebuffFrame = Dominos:CreateClass('Frame', Dominos.Frame)
local UNIT = "player"

--[[ Module Code ]]--

function DebuffModule:Load()
	self.frame = DebuffFrame:New()

end

function DebuffModule:Unload()
	self.frame:Free()
end


--[[ Debuff Frame Code ]]--

function DebuffFrame:New()
	local f = self.super.New(self, 'debuffs')

	if not self.debuffs then
		f:CreateDebuffs()
		f:SetFrameStrata('LOW')


	end

	f:LoadButtons()
	f:Layout()

	return f
end

function DebuffFrame:GetDefaults()
	return {
		scale = 1,
		point = 'CENTER',
		numButtons = NUM_DEBUFFS,
		y = 0,
		x = 0,
	}
end

function DebuffFrame:NumButtons()
	return self.sets.numButtons or NUM_DEBUFFS
end

function DebuffFrame:AddButton(i)
	local b = self.debuffs[i]
	b:SetParent(self.header)
	self.buttons[i] = b
end

function DebuffFrame:RemoveButton(i)
	local b = self.buttons[i]
	b:SetParent(nil)
	b:Hide()
	self.buttons[i] = nil
end

function DebuffFrame:CreateDebuffs()
	if not self.debuffs then
		self.debuffs = {}
	end

	for i = 1, NUM_DEBUFFS do
		if not self.debuffs[i] then
			self.debuffs[i] = self:CreateDebuff(i)
		end
	end
end

function DebuffFrame:CreateDebuff(index)
	local frame, frameName;
	local selfName = "DominosDebuffs"
	local frameName = selfName.."Debuff"..index;
	local debuff = CreateFrame("Button", frameName, UIParent, "BuffButtonTemplate, TargetDebuffFrameTemplate")
	debuff:SetSize(34, 34)
 
	debuff.icon = _G[frameName.."Icon"]
	debuff.icon:SetTexCoord(.1,.9,.1,.9)



	debuff.cooldown = _G[frameName.."Cooldown"]
	self:SetScript("OnUpdate", function(self, elapsed)
		for i=1, self:NumButtons() do
			self:UpdateDebuff(i )
		end
	end)

	return debuff
end

function DebuffFrame:UpdateDebuff(index, elapsed)
	local debuff = self.debuffs[index]
	DebuffFrame:AuraButtonUpdate(debuff:GetName(), index, "HARMFUL")
	debuff.id = index
end


--[[ Menu Code ]]--

local function AddSizeSlider(p)
	local L = LibStub('AceLocale-3.0'):GetLocale('Dominos-Config')

	local size = p:NewSlider(L.Size, 1, 1, 1)

	size.OnShow = function(self)
		self:SetMinMaxValues(1, NUM_DEBUFFS)
		self:SetValue(self:GetParent().owner:NumButtons())
	end

	size.UpdateValue = function(self, value)
		self:GetParent().owner:SetNumButtons(value)
		_G[self:GetParent():GetName() .. L.Columns]:OnShow()
	end
end

local function AddLayoutPanel(menu)
	local p = menu:NewPanel(LibStub('AceLocale-3.0'):GetLocale('Dominos-Config').Layout)
	p:NewOpacitySlider()
	p:NewFadeSlider()
	p:NewScaleSlider()
	p:NewPaddingSlider()
	p:NewSpacingSlider()
	p:NewColumnsSlider()
	AddSizeSlider(p)
end

local function AddAdvancedLayout(self)
	self:AddAdvancedPanel()
end

function DebuffFrame:CreateMenu()
	local menu = Dominos:NewMenu(self.id)
	AddLayoutPanel(menu)
	AddAdvancedLayout(menu)
	self.menu = menu
end

function DebuffFrame:AuraButtonUpdate(buttonName, index, filter)
	local unit = UNIT;
	local name, rank, texture, count, debuffType, duration, expirationTime, _, _, shouldConsolidate = UnitAura(unit, index, filter);
 
	local debuffName = buttonName;
	local debuff = _G[debuffName];

	if ( not name ) then
		-- No debuff so hide it if it exists
		if ( debuff ) then
			debuff:Hide();
			debuff.duration:Hide();
		end
		return nil;
	else
 
		-- If button doesn't exist make it
		if ( not debuff ) then
				debuff = CreateFrame("Button", debuffName, self.header, "BuffButtonTemplate");
			debuff.parent = self.header;
		end
		-- Setup Debuff
		debuff:SetID(index);
		debuff.unit = unit;
		debuff.filter = filter;
		debuff:SetAlpha(1.0);
		debuff.exitTime = nil;
		debuff:Show();
 		frameBorder = _G[debuffName.."Border"]
		frameBorder:Hide()
		if ( duration > 0 and expirationTime ) then
			debuff.cooldown:Show()
			CooldownFrame_SetTimer(debuff.cooldown, expirationTime - duration, duration, 1)
		else
			debuff.cooldown:Hide()
		end
		if ( duration > 0 and expirationTime ) then
			if ( SHOW_BUFF_DURATIONS == "1" ) then
				debuff.duration:Show();
			else
				debuff.duration:Hide();
			end
			if ( not debuff.timeLeft ) then
				debuff.timeLeft = expirationTime - GetTime();
				debuff:SetScript("OnUpdate", AuraButton_OnUpdate);
			else
				debuff.timeLeft = expirationTime - GetTime();
			end		 
			debuff.expirationTime = expirationTime; 
		else
			debuff.duration:Hide();
			if ( debuff.timeLeft ) then
				debuff:SetScript("OnUpdate", nil);
			end
			debuff.timeLeft = nil;
		end
		local icon = _G[debuffName.."Icon"];
		icon:SetTexture(texture);
 
		if ( count > 1 ) then
			debuff.count:SetText(count);
		else
			debuff.count:SetText("");
		end
 
		if ( GameTooltip:IsOwned(debuff) ) then
			GameTooltip_SetDefaultAnchor(GameTooltip, debuff)
			GameTooltip:SetUnitAura(UNIT, index, filter);
		end
	end
	return 1;
end